#define TRIGGER_PIN 16
#define ECHO_PIN 17
#define MAX_DISTANCE 100
#define SERVO_PIN 23
#define LED_PIN     18
#define LED_COUNT   12
#define TONE_OUTPUT_PIN 14
#define TONE_PWM_CHANNEL 0
#define MIC_PIN 32

#define BLYNK_TEMPLATE_ID     "YOUR_BLYNK_TEMPLATE_ID"
#define BLYNK_TEMPLATE_NAME   "YOUR_BLYNK_TEMPLATE_NAME"
#define BLYNK_AUTH_TOKEN      "YOUR_BLYNK_AUTH_TOKEN"

#define BLYNK_PRINT Serial

#include <string.h>
#include <WiFi.h>
#include <WiFiClient.h>
#include <BlynkSimpleEsp32.h>
#include <ESP32Servo.h>
#include <Adafruit_NeoPixel.h>

Adafruit_NeoPixel strip(LED_COUNT, LED_PIN, NEO_GRB + NEO_KHZ800);

#include "formChecker.h"
#include "plankTimer.h"
#include "pitches.h"
#include "breathDetection.h"
#include "repsCounter.h"
#include "WiFiConnection.h"
#include "WeatherFetcher.h"
#include "Config.h"

WiFiConnection wifi;

bool isResting = false;
bool inProgress = false;
int irSendCount = 0;
bool shouldSendIR = false;
unsigned long lastIRSendTime = 0;
int reps = 0;

FormChecker formCheck(TRIGGER_PIN, ECHO_PIN, MAX_DISTANCE);
PlankTimer plankTimer;
BreathDetector breathDetector(MIC_PIN, 4000, LED_COUNT);
RepsCounter repsCounter(7500, 200);

Servo myservo;

BLYNK_WRITE(V1) {
  int switchValue = param.asInt();
  if (switchValue == 0) {
    isResting = false;
  } else {
    isResting = true;
  }
}

BLYNK_WRITE(V5) {
  int switchValue = param.asInt();
  Serial.println("Change in state");
  if (switchValue == 0) {
    myservo.write(0);
    inProgress = false;
  } else {
    myservo.write(180);
    inProgress = true;
  }
}

BLYNK_WRITE(V6) {
  plankTimer.setDuration(param.asInt());
}

BLYNK_WRITE(V7) {
  if (param.asInt() == 1) {
    plankTimer.start();
  }
}

void setup() {
  Serial.begin(115200);
  Blynk.begin(BLYNK_AUTH_TOKEN, ssid, password);
  ESP32PWM::allocateTimer(0);
  // standard 50 hz servo
  myservo.setPeriodHertz(50);
  myservo.attach(SERVO_PIN, 500, 2500);
  strip.begin();
  strip.show();
  ledcAttachPin(TONE_OUTPUT_PIN, TONE_PWM_CHANNEL);
  wifi.connectToWiFi();
  IrSender.begin(IR_SEND_PIN);
  repsCounter.begin();
}

void loop() {
  Blynk.run();
  formCheck.update();

  if (formCheck.isFormBad() && !isResting) {
    Serial.println("Bad");
    Blynk.logEvent("form_check");
  }
  
  plankTimer.update();
  breathDetector.update();

  unsigned long currentTime = millis();

  if (currentTime - lastCheckTime >= checkInterval || lastCheckTime == 0) {
    lastCheckTime = currentTime;
    float temp = getCurrentTemperature();
    Serial.print("Current temperature: ");
    Serial.println(temp);
    if (temp > 22.0) {
      Serial.println("Temperature is above 22°C. Preparing to send IR...");
      shouldSendIR = true;
      irSendCount = 0;
      lastIRSendTime = 0;
    } else {
      Serial.println("Temperature is 22°C or below. No signal sent.");
    }
  }

  if (shouldSendIR && irSendCount < numRepeats) {
    if (lastIRSendTime == 0 || currentTime - lastIRSendTime >= irInterval) {
      IrSender.sendNEC(0x4, 0x8, 3);
      Serial.print("Sent IR signal ");
      Serial.println(irSendCount + 1);
      irSendCount++;
      lastIRSendTime = currentTime;
    }
  } else if (irSendCount >= numRepeats) {
    shouldSendIR = false;
  }

  repsCounter.update();
  int currentReps = repsCounter.getRepsCount();
  if (currentReps > reps) {
    reps = currentReps;
    Blynk.virtualWrite(V2, reps);
  }
}
